@@ -10,11 +10,9 @@
 #include "breezedecoration.h"
 
 #include "breezesettingsprovider.h"
-#include "config-breeze.h"
 #include "config/breezeconfigwidget.h"
 
 #include "breezebutton.h"
-#include "breezesizegrip.h"
 
 #include "breezeboxshadowrenderer.h"
 
@@ -31,17 +29,10 @@
 #include <QDBusPendingCallWatcher>
 #include <QDBusPendingReply>
 #include <QPainter>
+#include <QPainterPath>
 #include <QTextStream>
 #include <QTimer>
 
-#if BREEZE_HAVE_X11
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
-#include <private/qtx11extras_p.h>
-#else
-#include <QX11Info>
-#endif
-#endif
-
 #include <cmath>
 
 K_PLUGIN_FACTORY_WITH_JSON(BreezeDecoFactory, "breeze.json", registerPlugin<Breeze::Decoration>(); registerPlugin<Breeze::Button>();
@@ -134,6 +125,8 @@ static int g_shadowStrength = 255;
 static QColor g_shadowColor = Qt::black;
 static QSharedPointer<KDecoration2::DecorationShadow> g_sShadow;
 static QSharedPointer<KDecoration2::DecorationShadow> g_sShadowInactive;
+static int g_lastBorderSize;
+static QColor g_lastOutlineColor;
 
 //________________________________________________________________
 Decoration::Decoration(QObject *parent, const QVariantList &args)
@@ -152,32 +145,29 @@ Decoration::~Decoration()
         // last deco destroyed, clean up shadow
         g_sShadow.clear();
     }
-
-    deleteSizeGrip();
 }
 
 //________________________________________________________________
 void Decoration::setOpacity(qreal value)
 {
-    if (m_opacity == value)
+    if (m_opacity == value) {
         return;
+    }
     m_opacity = value;
     update();
-
-    if (m_sizeGrip)
-        m_sizeGrip->update();
 }
 
 //________________________________________________________________
 QColor Decoration::titleBarColor() const
 {
     const auto c = client().toStrongRef();
-    if (hideTitleBar())
+    if (hideTitleBar()) {
         return c->color(ColorGroup::Inactive, ColorRole::TitleBar);
-    else if (m_animation->state() == QAbstractAnimation::Running) {
+    } else if (m_animation->state() == QAbstractAnimation::Running) {
         return KColorUtils::mix(c->color(ColorGroup::Inactive, ColorRole::TitleBar), c->color(ColorGroup::Active, ColorRole::TitleBar), m_opacity);
-    } else
+    } else {
         return c->color(c->isActive() ? ColorGroup::Active : ColorGroup::Inactive, ColorRole::TitleBar);
+    }
 }
 
 //________________________________________________________________
@@ -186,8 +176,9 @@ QColor Decoration::fontColor() const
     const auto c = client().toStrongRef();
     if (m_animation->state() == QAbstractAnimation::Running) {
         return KColorUtils::mix(c->color(ColorGroup::Inactive, ColorRole::Foreground), c->color(ColorGroup::Active, ColorRole::Foreground), m_opacity);
-    } else
+    } else {
         return c->color(c->isActive() ? ColorGroup::Active : ColorGroup::Inactive, ColorRole::Foreground);
+    }
 }
 
 //________________________________________________________________
@@ -308,8 +299,9 @@ void Decoration::updateAnimationState()
         const auto c = client().toStrongRef();
         m_shadowAnimation->setDirection(c->isActive() ? QAbstractAnimation::Forward : QAbstractAnimation::Backward);
         m_shadowAnimation->setEasingCurve(c->isActive() ? QEasingCurve::OutCubic : QEasingCurve::InCubic);
-        if (m_shadowAnimation->state() != QAbstractAnimation::Running)
+        if (m_shadowAnimation->state() != QAbstractAnimation::Running) {
             m_shadowAnimation->start();
+        }
 
     } else {
         updateShadow();
@@ -318,23 +310,15 @@ void Decoration::updateAnimationState()
     if (m_animation->duration() > 0) {
         const auto c = client().toStrongRef();
         m_animation->setDirection(c->isActive() ? QAbstractAnimation::Forward : QAbstractAnimation::Backward);
-        if (m_animation->state() != QAbstractAnimation::Running)
+        if (m_animation->state() != QAbstractAnimation::Running) {
             m_animation->start();
+        }
 
     } else {
         update();
     }
 }
 
-//________________________________________________________________
-void Decoration::updateSizeGripVisibility()
-{
-    const auto c = client().toStrongRef();
-    if (m_sizeGrip) {
-        m_sizeGrip->setVisible(c->isResizeable() && !isMaximized() && !c->isShaded());
-    }
-}
-
 //________________________________________________________________
 int Decoration::borderSize(bool bottom) const
 {
@@ -412,12 +396,6 @@ void Decoration::reconfigure()
 
     // shadow
     updateShadow();
-
-    // size grip
-    if (hasNoBorders() && m_internalSettings->drawSizeGrip())
-        createSizeGrip();
-    else
-        deleteSizeGrip();
 }
 
 //________________________________________________________________
@@ -432,9 +410,9 @@ void Decoration::recalculateBorders()
     const int bottom = (c->isShaded() || isBottomEdge()) ? 0 : borderSize(true);
 
     int top = 0;
-    if (hideTitleBar())
+    if (hideTitleBar()) {
         top = bottom;
-    else {
+    } else {
         QFontMetrics fm(s->font());
         top += qMax(fm.height(), buttonHeight());
 
@@ -454,16 +432,21 @@ void Decoration::recalculateBorders()
     int extSides = 0;
     int extBottom = 0;
     if (hasNoBorders()) {
-        if (!isMaximizedHorizontally())
+        if (!isMaximizedHorizontally()) {
             extSides = extSize;
-        if (!isMaximizedVertically())
+        }
+        if (!isMaximizedVertically()) {
             extBottom = extSize;
+        }
 
     } else if (hasNoSideBorders() && !isMaximizedHorizontally()) {
         extSides = extSize;
     }
 
     setResizeOnlyBorders(QMargins(extSides, 0, extSides, extBottom));
+
+    // Update shadows and clear outline to make sure outline changes when borders are changed
+    updateShadow();
 }
 
 //________________________________________________________________
@@ -512,8 +495,9 @@ void Decoration::updateButtonsGeometry()
 
             m_leftButtons->setPos(QPointF(0, vPadding));
 
-        } else
+        } else {
             m_leftButtons->setPos(QPointF(hPadding + borderLeft(), vPadding));
+        }
     }
 
     // right buttons
@@ -531,8 +515,9 @@ void Decoration::updateButtonsGeometry()
 
             m_rightButtons->setPos(QPointF(size().width() - m_rightButtons->geometry().width(), vPadding));
 
-        } else
+        } else {
             m_rightButtons->setPos(QPointF(size().width() - m_rightButtons->geometry().width() - hPadding - borderRight(), vPadding));
+        }
     }
 
     update();
@@ -554,19 +539,22 @@ void Decoration::paint(QPainter *painter, const QRect &repaintRegion)
         painter->setBrush(c->color(c->isActive() ? ColorGroup::Active : ColorGroup::Inactive, ColorRole::Frame));
 
         // clip away the top part
-        if (!hideTitleBar())
+        if (!hideTitleBar()) {
             painter->setClipRect(0, borderTop(), size().width(), size().height() - borderTop(), Qt::IntersectClip);
+        }
 
-        if (s->isAlphaChannelSupported())
+        if (s->isAlphaChannelSupported()) {
             painter->drawRoundedRect(rect(), m_scaledCornerRadius, m_scaledCornerRadius);
-        else
+        } else {
             painter->drawRect(rect());
+        }
 
         painter->restore();
     }
 
-    if (!hideTitleBar())
+    if (!hideTitleBar()) {
         paintTitleBar(painter, repaintRegion);
+    }
 
     if (hasBorders() && !s->isAlphaChannelSupported()) {
         painter->save();
@@ -583,14 +571,18 @@ void Decoration::paint(QPainter *painter, const QRect &repaintRegion)
 void Decoration::paintTitleBar(QPainter *painter, const QRect &repaintRegion)
 {
     const auto c = client().toStrongRef();
-    const QRect frontRect(QPoint(0, 1), QSize(size().width(), borderTop() + 1));
-    const QRect backRect(QPoint(0, 0), QSize(size().width(), borderTop()));
+    // Check if the window is maximized and if so, remove the extra gap that makes room for outlines,
+    // since they're not even drawn when window is maximized
+    const int outlineGap = isMaximized() ? 0 : 1;
+    const QRect frontRect(QPoint(0, 1), QSize(size().width(), borderTop()));
+    const QRect backRect(QPoint(0, outlineGap), QSize(size().width(), borderTop() - outlineGap));
 
     QBrush frontBrush;
-    QBrush backBrush(this->titleBarColor().lighter(130));
+    QBrush backBrush(this->titleBarColor());
 
-    if (!backRect.intersects(repaintRegion))
+    if (!backRect.intersects(repaintRegion)) {
         return;
+    }
 
     painter->save();
     painter->setPen(Qt::NoPen);
@@ -693,9 +685,9 @@ int Decoration::captionHeight() const
 //________________________________________________________________
 QPair<QRect, Qt::Alignment> Decoration::captionRect() const
 {
-    if (hideTitleBar())
+    if (hideTitleBar()) {
         return qMakePair(QRect(), Qt::AlignCenter);
-    else {
+    } else {
         auto c = client().toStrongRef();
         const int leftOffset = m_leftButtons->buttons().isEmpty()
             ? Metrics::TitleBar_SideMargin * settings()->smallSpacing()
@@ -729,12 +721,13 @@ QPair<QRect, Qt::Alignment> Decoration::captionRect() const
             boundingRect.setHeight(captionHeight());
             boundingRect.moveLeft((size().width() - boundingRect.width()) / 2);
 
-            if (boundingRect.left() < leftOffset)
+            if (boundingRect.left() < leftOffset) {
                 return qMakePair(maxRect, Qt::AlignVCenter | Qt::AlignLeft);
-            else if (boundingRect.right() > size().width() - rightOffset)
+            } else if (boundingRect.right() > size().width() - rightOffset) {
                 return qMakePair(maxRect, Qt::AlignVCenter | Qt::AlignRight);
-            else
+            } else {
                 return qMakePair(fullRect, Qt::AlignCenter);
+            }
         }
         }
     }
@@ -744,9 +737,20 @@ QPair<QRect, Qt::Alignment> Decoration::captionRect() const
 void Decoration::updateShadow()
 {
     auto s = settings();
+    auto c = client().toStrongRef();
+    auto outlineColor = c->color(c->isActive() ? ColorGroup::Active : ColorGroup::Inactive, ColorRole::TitleBar);
+    auto backgroundColor = c->color(c->isActive() ? ColorGroup::Active : ColorGroup::Inactive, ColorRole::Frame);
+    // Bind lightness between 0.1 and 1.0 so it can never be completely black.
+    // Outlines only have transparency if alpha channel is supported
+    outlineColor.setHslF(outlineColor.hslHueF(),
+                         outlineColor.hslSaturationF(),
+                         qBound(0.1, outlineColor.lightnessF(), 1.0),
+                         s->isAlphaChannelSupported() ? 0.9 : 1.0);
+    outlineColor.lightnessF() >= 0.5 ? outlineColor = outlineColor.darker(170) : outlineColor = outlineColor.lighter(170);
+
     // Animated case, no cached shadow object
     if ((m_shadowAnimation->state() == QAbstractAnimation::Running) && (m_shadowOpacity != 0.0) && (m_shadowOpacity != 1.0)) {
-        setShadow(createShadowObject(0.5 + m_shadowOpacity * 0.5));
+        setShadow(createShadowObject(0.5 + m_shadowOpacity * 0.5, outlineColor));
         return;
     }
 
@@ -759,20 +763,25 @@ void Decoration::updateShadow()
         g_shadowColor = m_internalSettings->shadowColor();
     }
 
-    auto c = client().toStrongRef();
     auto &shadow = (c->isActive()) ? g_sShadow : g_sShadowInactive;
-    if (!shadow) {
-        shadow = createShadowObject(c->isActive() ? 1.0 : 0.5);
+    if (!shadow || g_lastBorderSize != borderSize(true) || g_lastOutlineColor != outlineColor) {
+        // Update both active and inactive shadows so outline stays consistent between the two
+        g_sShadow = createShadowObject(1.0, outlineColor);
+        g_sShadowInactive = createShadowObject(0.5, outlineColor);
+        g_lastBorderSize = borderSize(true);
+        g_lastOutlineColor = outlineColor;
     }
     setShadow(shadow);
 }
 
 //________________________________________________________________
-QSharedPointer<KDecoration2::DecorationShadow> Decoration::createShadowObject(const float strengthScale)
+QSharedPointer<KDecoration2::DecorationShadow> Decoration::createShadowObject(const float strengthScale, const QColor &outlineColor)
 {
-    const CompositeShadowParams params = lookupShadowParams(m_internalSettings->shadowSize());
+    CompositeShadowParams params = lookupShadowParams(m_internalSettings->shadowSize());
     if (params.isNone()) {
-        return nullptr;
+        // If shadows are disabled, set shadow opacity to 0.
+        // This allows the outline effect to show up without the shadow effect.
+        params = CompositeShadowParams(QPoint(0, 4), ShadowParams(QPoint(0, 0), 16, 0), ShadowParams(QPoint(0, -2), 8, 0));
     }
 
     auto withOpacity = [](const QColor &color, qreal opacity) -> QColor {
@@ -814,11 +823,38 @@ QSharedPointer<KDecoration2::DecorationShadow> Decoration::createShadowObject(co
     painter.setCompositionMode(QPainter::CompositionMode_DestinationOut);
     painter.drawRoundedRect(innerRect, m_scaledCornerRadius + 0.5, m_scaledCornerRadius + 0.5);
 
-    // Draw outline.
-    painter.setPen(withOpacity(Qt::black, 0.1));
+    // Draw window outline
+    const qreal outlineWidth = 1.001;
+    const qreal penOffset = outlineWidth / 2;
+
+    // Titlebar already has an outline, so move the top of the outline on the same level to avoid 2px width on top outline.
+    QRectF outlineRect = innerRect + QMarginsF(penOffset, -penOffset, penOffset, penOffset);
+    qreal cornerSize = m_scaledCornerRadius * 2;
+    QRectF cornerRect(outlineRect.x(), outlineRect.y(), cornerSize, cornerSize);
+    QPainterPath outlinePath;
+
+    outlinePath.arcMoveTo(cornerRect, 180);
+    outlinePath.arcTo(cornerRect, 180, -90);
+    cornerRect.moveTopRight(outlineRect.topRight());
+    outlinePath.arcTo(cornerRect, 90, -90);
+
+    // Check if border size is "no borders" or "no side-borders"
+    if (borderSize(true) == 0) {
+        outlinePath.lineTo(outlineRect.bottomRight());
+        outlinePath.lineTo(outlineRect.bottomLeft());
+    } else {
+        cornerRect.moveBottomRight(outlineRect.bottomRight());
+        outlinePath.arcTo(cornerRect, 0, -90);
+        cornerRect.moveBottomLeft(outlineRect.bottomLeft());
+        outlinePath.arcTo(cornerRect, 270, -90);
+    }
+    outlinePath.closeSubpath();
+
+    painter.setPen(QPen(outlineColor, outlineWidth));
     painter.setBrush(Qt::NoBrush);
-    painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
-    painter.drawRoundedRect(innerRect, m_scaledCornerRadius - 0.5, m_scaledCornerRadius - 0.5);
+    painter.setCompositionMode(QPainter::CompositionMode_Source);
+    painter.setRenderHint(QPainter::Antialiasing);
+    painter.drawPath(outlinePath);
 
     painter.end();
 
@@ -829,40 +865,6 @@ QSharedPointer<KDecoration2::DecorationShadow> Decoration::createShadowObject(co
     return ret;
 }
 
-//_________________________________________________________________
-void Decoration::createSizeGrip()
-{
-    // do nothing if size grip already exist
-    if (m_sizeGrip)
-        return;
-
-#if BREEZE_HAVE_X11
-    if (!QX11Info::isPlatformX11())
-        return;
-
-    // access client
-    auto c = client().toStrongRef();
-    if (!c)
-        return;
-
-    if (c->windowId() != 0) {
-        m_sizeGrip = new SizeGrip(this);
-        connect(c.data(), &KDecoration2::DecoratedClient::maximizedChanged, this, &Decoration::updateSizeGripVisibility);
-        connect(c.data(), &KDecoration2::DecoratedClient::shadedChanged, this, &Decoration::updateSizeGripVisibility);
-        connect(c.data(), &KDecoration2::DecoratedClient::resizeableChanged, this, &Decoration::updateSizeGripVisibility);
-    }
-#endif
-}
-
-//_________________________________________________________________
-void Decoration::deleteSizeGrip()
-{
-    if (m_sizeGrip) {
-        m_sizeGrip->deleteLater();
-        m_sizeGrip = nullptr;
-    }
-}
-
 void Decoration::setScaledCornerRadius()
 {
     m_scaledCornerRadius = Metrics::Frame_FrameRadius * settings()->smallSpacing();
