@@ -86,9 +86,6 @@ public:
     //* application polishing
     void polish(QApplication *) override;
 
-    //* application unpolishing
-    void unpolish(QApplication *) override;
-
     //* polish scrollarea
     void polishScrollArea(QAbstractScrollArea *);
 
@@ -346,9 +343,6 @@ private:
     //* spinbox arrows
     void renderSpinBoxArrow(const SubControl &, const QStyleOptionSpinBox *, QPainter *, const QWidget *) const;
 
-    //* menu title
-    void renderMenuTitle(const QStyleOptionToolButton *, QPainter *, const QWidget *) const;
-
     //* return dial angle based on option and value
     qreal dialAngle(const QStyleOptionSlider *, int) const;
 
@@ -492,9 +486,6 @@ private:
     //* return true if icons should be shown on buttons
     bool showIconsOnPushButtons() const;
 
-    //* return true if passed widget is a menu title (KMenu::addTitle)
-    bool isMenuTitle(const QWidget *) const;
-
     //* return true if passed widget is a menu title (KMenu::addTitle)
     bool hasAlteredBackground(const QWidget *) const;
 
@@ -580,37 +571,43 @@ private:
 bool Style::preceeds(const QPoint &point, const QRect &bound, const QStyleOption *option) const
 {
     if (option->state & QStyle::State_Horizontal) {
-        if (option->direction == Qt::LeftToRight)
+        if (option->direction == Qt::LeftToRight) {
             return point.x() < bound.right();
-        else
+        } else {
             return point.x() > bound.x();
+        }
 
-    } else
+    } else {
         return point.y() < bound.y();
+    }
 }
 
 //_________________________________________________________________________
 QStyle::SubControl Style::scrollBarHitTest(const QRect &rect, const QPoint &point, const QStyleOption *option) const
 {
     if (option->state & QStyle::State_Horizontal) {
-        if (option->direction == Qt::LeftToRight)
+        if (option->direction == Qt::LeftToRight) {
             return point.x() < rect.center().x() ? QStyle::SC_ScrollBarSubLine : QStyle::SC_ScrollBarAddLine;
-        else
+        } else {
             return point.x() > rect.center().x() ? QStyle::SC_ScrollBarSubLine : QStyle::SC_ScrollBarAddLine;
+        }
 
-    } else
+    } else {
         return point.y() < rect.center().y() ? QStyle::SC_ScrollBarSubLine : QStyle::SC_ScrollBarAddLine;
+    }
 }
 
 //_________________________________________________________________________
 bool Style::hasParent(const QWidget *widget, const char *className) const
 {
-    if (!widget)
+    if (!widget) {
         return false;
+    }
 
     while ((widget = widget->parentWidget())) {
-        if (widget->inherits(className))
+        if (widget->inherits(className)) {
             return true;
+        }
     }
 
     return false;
@@ -620,12 +617,14 @@ bool Style::hasParent(const QWidget *widget, const char *className) const
 template<typename T>
 bool Style::hasParent(const QWidget *widget) const
 {
-    if (!widget)
+    if (!widget) {
         return false;
+    }
 
     while ((widget = widget->parentWidget())) {
-        if (qobject_cast<const T *>(widget))
+        if (qobject_cast<const T *>(widget)) {
             return true;
+        }
     }
 
     return false;
